from tests import IntegrationTest

class TestCreateAnswer(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru2_id, [self.instru1_id])['user']['id']
        self.date1_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate', end_time='18:00:00')['id']
        
    def test_create_answer(self):
        _json = self.create_answer(self.user1_id, self.date1_id)
        self.assertAnswer(_json, 1, self.user1_id, self.date1_id, True)
        
    def test_create_answer_dupe_error(self):
        _json = self.create_answer(self.user1_id, self.date1_id)
        self.assertAnswer(_json, 1, self.user1_id, self.date1_id, True)
        _rs = self.create_answer(self.user1_id, self.date1_id, fail=True)
        self.assertEqual(409, _rs.status_code)

    def test_create_answer_error(self):
        _rs = self.create_answer(self.user1_id, 99, fail=True)
        self.assertEqual(400, _rs.status_code)


from tests import IntegrationTest

class TestCreateUser(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.create_user('user1', self.instru1_id, [])
        self.create_user('user3', self.instru2_id, [])
        
    def test_create_user_conflict(self):
        _json = self.get_users()
        self.assertEqual(2, len(_json['users']))
        _rs = self.create_user('user1', self.instru1_id, [], fail=True)
        self.assertEqual(409, _rs.status_code)
        
    def test_create_user_conflict_case_insensitive(self):
        _json = self.get_users()
        self.assertEqual(2, len(_json['users']))
        _rs = self.create_user('User1', self.instru1_id, [], fail=True)
        self.assertEqual(409, _rs.status_code)

    def test_create_user_valid(self):
        # Pre-check
        _json = self.get_users()
        self.assertEqual(2, len(_json['users']))
        
        # Create user
        _json = self.create_user('user2', self.instru2_id, [self.instru3_id])
        self.assertEqual(3, _json['user']['id'])
        self.assertEqual('user2', _json['user']['name'])
        self.assertInstrument(_json['main_instrument'], self.instru2_id, self.INSTRU2, self.INSTRU2_RANK)
        self.assertEqual(1, len(_json['instruments']))
        self.assertInstruments(_json['instruments'], 0, self.instru3_id, self.INSTRU3, self.INSTRU3_RANK)
        
        # Post check
        _json = self.get_users()
        self.assertEqual(3, len(_json['users']))
        self.assertUsers(_json['users'], 0, 1, 'user1')
        self.assertUsers(_json['users'], 1, 3, 'user2')
        self.assertUsers(_json['users'], 2, 2, 'user3')
        
    def test_create_user_dupe_instrument(self):
        user4_id = self.create_user('user4', self.instru1_id, [self.instru1_id])['user']['id']
        self.assertEqual(0, len(self.get_user(user4_id, False)['instruments']))
        
    def test_create_user_not_existing_instrument(self):
        _rs = self.create_user('user4', 99, [], fail=True)
        self.assertEqual(400, _rs.status_code)


#! /usr/bin/python3
import argparse
import sys
import flask
import json
import logging
import logging.handlers
import os

from flask_cors import CORS
from flask_restful import Api, Resource
from functools import wraps
from typing import Any

from popoll_backend.model.payload.history import History
from popoll_backend.query.all.upload_polls import UploadPolls
from popoll_backend.query.poll.create_answer import CreateAnswer
from popoll_backend.query.poll.create_date import CreateDate
from popoll_backend.query.poll.create_poll import CreatePoll
from popoll_backend.query.poll.create_user import CreateUser
from popoll_backend.query.poll.delete_answer import DeleteAnswer
from popoll_backend.query.poll.delete_date import DeleteDate
from popoll_backend.query.all.delete_old_dates import DeleteOldDates
from popoll_backend.query.poll.delete_user import DeleteUser
from popoll_backend.query.poll.get_all_instruments import GetAllInstruments
from popoll_backend.query.all.get_all_sessions import GetAllSession
from popoll_backend.query.poll.get_answer import GetAnswer
from popoll_backend.query.poll.get_date import GetDate
from popoll_backend.query.poll.get_date_details import GetDateDetails
from popoll_backend.query.poll.get_dates import GetDates
from popoll_backend.query.poll.get_instruments import GetInstruments
from popoll_backend.query.poll.get_poll import GetPoll
from popoll_backend.query.poll.get_search_answer import GetSearchAnswer
from popoll_backend.query.poll.get_session import GetSession
from popoll_backend.query.poll.get_user_details import GetUserDetails
from popoll_backend.query.poll.get_user_with_instruments import GetUserWithInstruments
from popoll_backend.query.poll.get_users import GetUsers
from popoll_backend.query.poll.update_answer import UpdateAnswer
from popoll_backend.query.poll.update_date import UpdateDate
from popoll_backend.query.poll.update_poll import UpdatePoll
from popoll_backend.query.poll.create_session import CreateSession
from popoll_backend.query.poll.update_user import UpdateUser


app = flask.Flask(__name__)
CORS(app)
api = Api(app)


def body(request: flask.request, param: str, default: Any=None, mandatory=True):
    if request.data != None:
        _body = json.loads(request.data)
        return _body[param] if mandatory else _body.get(param, default)
    else:
        return flask.abort(400, f'Missing parameter [{param}]') if mandatory else default

def queryParam(request: flask.request, param: str) -> bool:
    return request.args.get(param, '') == 'true'

def history(f):
    @wraps(f)
    def decorated(*args, **kwargs):
        _poll = kwargs.get('poll')
        res = f(*args, **kwargs)
        os.makedirs(os.path.join('.history', _poll), exist_ok=True)
        logger = logging.getLogger('my_logger')
        logger.handlers.clear()
        handler = logging.handlers.RotatingFileHandler(os.path.join('.history', _poll, f'{_poll}.history.log'), maxBytes=1073741824, backupCount=10)
        logger.addHandler(handler)
        logger.warning(json.dumps(History(flask.request, res, kwargs).toJSON()))
        return res
    return decorated





class PollPollEndpoint(Resource):
    def get(self, poll: str) -> str: return GetPoll(poll).run().toJSON()
    
    @history
    def post(self, poll:str) -> str: return CreatePoll(poll, body(flask.request, 'name', mandatory=False, default=poll), body(flask.request, 'instruments', mandatory=False, default=[]), body(flask.request, 'color', mandatory=False, default="#000000")).run().toJSON()
    
    @history
    def put(self, poll:str) -> str: return UpdatePoll(poll, body(flask.request, 'name'), body(flask.request, 'color')).run().toJSON()
    
    
    

class PollInstrumentsEndpoint(Resource):
    def get(self, poll: str) -> str: 
        if queryParam(flask.request, 'used_only'):
            return GetInstruments(poll).run().toJSON()
        else:
            return GetAllInstruments(poll).run().toJSON()
    
    






class PollUsersEndpoint(Resource):
    def get(self, poll: str) -> str: return GetUsers(poll).run().toJSON()
    
    @history
    def post(self, poll: str) -> str: return CreateUser(poll, body(flask.request, 'user')['name'], body(flask.request, 'main_instrument')['id'], [i['id'] for i in body(flask.request, 'instruments')]).run().toJSON()



class PollUserEndpoint(Resource):
    def get(self, poll: str, id:int) -> str: 
        if queryParam(flask.request, 'details'):
            return GetUserDetails(poll, id).run().toJSON()
        else:
            return GetUserWithInstruments(poll, id).run().toJSON()
    
    @history
    def put(self, poll: str, id: int) -> str: return UpdateUser(poll, id, body(flask.request, 'user')['name'], body(flask.request, 'main_instrument')['id'], [i['id'] for i in body(flask.request, 'instruments')]).run().toJSON()
    
    @history
    def delete(self, poll: str, id: int) -> str: return DeleteUser(poll, id).run().toJSON()







class PollDatesEndpoint(Resource):
    def get(self, poll: str) -> str: return GetDates(poll).run().toJSON()
    
    @history
    def post(self, poll: str) -> str: return CreateDate(
        poll, 
        body(flask.request, 'title'), 
        body(flask.request, 'date'), 
        body(flask.request, 'time', mandatory=False), 
        body(flask.request, 'end_time', mandatory=False),
        body(flask.request, 'is_frozen'), 
    ).run().toJSON()



class PollDateEndpoint(Resource):
    def get(self, poll: str, id:int) -> str:
        if queryParam(flask.request, 'details'):
            return GetDateDetails(poll, id).run().toJSON()
        else:
            return GetDate(poll, id).run().toJSON()
    
    @history
    def put(self, poll: str, id: int) -> str: return UpdateDate(
        poll, 
        id, 
        body(flask.request, 'title'), 
        body(flask.request, 'date'), 
        body(flask.request, 'time', mandatory=False), 
        body(flask.request, 'end_time', mandatory=False),
        body(flask.request, 'is_frozen')
    ).run().toJSON()
    
    @history
    def delete(self, poll: str, id: int) -> str: return DeleteDate(poll, id).run().toJSON()







class PollAnswersEndpoint(Resource):
    
    @history
    def post(self, poll: str) -> str: return CreateAnswer(poll, body(flask.request, 'user_id'), body(flask.request, 'date_id')).run().toJSON()


class PollAnswerEndpoint(Resource):
    
    def get(self, poll: str, id:int) -> str: return GetAnswer(poll, id).run().toJSON()
    
    @history
    def put(self, poll: str, id: int) -> str: return UpdateAnswer(poll, id, body(flask.request, 'response')).run().toJSON()
    
    @history
    def delete(self, poll: str, id: int) -> str: return DeleteAnswer(poll, id).run().toJSON()

class PollGetAnswerEndpoint(Resource):
    def get(self, poll: str, userId: int, dateId: int) -> str: return GetSearchAnswer(poll, userId, dateId).run().toJSON()


class PollSessionEndpoint(Resource):
    def get(self, poll: str, id: str) -> str: return GetSession(poll, id).run().toJSON()
    
    @history
    def post(self, poll: str, id: str) -> str: return CreateSession(poll, id, body(flask.request, 'user_id')).run().toJSON()




class SessionEndpoint(Resource):
    def get(self, id: str) -> str: return GetAllSession(id).run().toJSON()


class DatesEndpoint(Resource):
    def delete(self) -> str: return DeleteOldDates().run().toJSON()
    
class PollsEndpoint(Resource):
    def post(self) -> str: return UploadPolls().run().toJSON()


api.add_resource(SessionEndpoint, '/session/<string:id>')
api.add_resource(DatesEndpoint, '/date')
api.add_resource(PollsEndpoint, '/poll')

api.add_resource(PollPollEndpoint, '/<string:poll>')
api.add_resource(PollInstrumentsEndpoint, '/<string:poll>/instrument')
api.add_resource(PollUsersEndpoint, '/<string:poll>/user')
api.add_resource(PollUserEndpoint, '/<string:poll>/user/<int:id>')
api.add_resource(PollDatesEndpoint, '/<string:poll>/date')
api.add_resource(PollDateEndpoint, '/<string:poll>/date/<int:id>')
api.add_resource(PollAnswersEndpoint, '/<string:poll>/answer')
api.add_resource(PollAnswerEndpoint, '/<string:poll>/answer/<int:id>')
api.add_resource(PollGetAnswerEndpoint, '/<string:poll>/answer/<int:userId>/<int:dateId>')
api.add_resource(PollSessionEndpoint, '/<string:poll>/session/<string:id>')


def get_options():
    parser = argparse.ArgumentParser()
    parser.add_argument('--host', help='the hostname to listen on', default='0.0.0.0')
    parser.add_argument('--port', help='the port of the webserver', default=4444)
    parser.add_argument('--debug', help='Enable debugging', action='store_true')
    return parser

def run(args):
    app.run(debug=args.debug, host=args.host, port=args.port)

if __name__ == '__main__':
    args = get_options().parse_args()
    run(args)



from typing import List, Optional
from popoll_backend.model.db.answer import Answer
from popoll_backend.model.db.date import Date
from popoll_backend.model import Payload
from popoll_backend.model.db.user import User

class UserDetailsDate():
    
    def __init__(self, user: User, date: Date, answers: List[Answer]):
        self.date: Date = date
        _ans = [a for a in answers if a.user_id == user.id and a.date_id == date.id]
        self.answer: Optional[Answer] = _ans[0] if len(_ans) > 0 else None
        

class UserDetails(Payload):
    
    def __init__(self, user: User, answers: List[Answer], dates: List[Date]):
        self.user: User = user
        self.dates: List[UserDetailsDate] = [UserDetailsDate(user, date, answers) for date in dates]


import contextlib
import os
from tests import DB_NAME, IntegrationTest

class TestCreatePoll(IntegrationTest):
    
    DB_NAME_2 = 'test_integration_2'
    
    def test_db_creation(self):
        _json = self.get_instruments(False)
        
        self.assertEqual(self.instru1_id, _json['instruments'][0]['id'])
        self.assertEqual(self.INSTRU1, _json['instruments'][0]['name'])
        self.assertEqual(self.instru2_id, _json['instruments'][1]['id'])
        self.assertEqual(self.INSTRU2, _json['instruments'][1]['name'])
        self.assertEqual(self.instru3_id, _json['instruments'][2]['id'])
        self.assertEqual(self.INSTRU3, _json['instruments'][2]['name'])
        
    def test_can_create_another_db(self):
        _json = self.create_poll(self.DB_NAME_2, self.DB_NAME_2, '#ff8b00')
        self.assertEqual(0, _json['id'])

    def test_no_duplicate_poll_id(self):
        rs = self.create_poll(DB_NAME, self.DB_NAME_2, '#ff8b00', fail=True)
        self.assertEqual(409, rs.status_code)
        
    def tearDown(self):
        self.tearDown_db(self.DB_NAME_2)


from tests import IntegrationTest

class TestDeleteUser(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru1_id, [])['user']['id']
        self.user2_id = self.create_user('user2', self.instru1_id, [])['user']['id']
        
    def test_delete_user(self):
        _json = self.delete_user(self.user1_id)
        self.assertEqual({}, _json)

    def test_delete_user_invalid(self):
        _rs = self.delete_user(99, fail=True)
        self.assertEqual(200, _rs.status_code)
        self.assertEqual({}, _rs.json)


from tests import IntegrationTest

class TestDeleteAnswer(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru2_id, [self.instru1_id])['user']['id']
        self.date1_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate', end_time='18:00:00')['id']
        self.answer1_id = self.create_answer(self.user1_id, self.date1_id)['id']
        
    def test_delete_answer(self):
        self.delete_answer(self.answer1_id)
        _rs = self.get_answer(self.answer1_id, fail=True)
        self.assertEqual(404, _rs.status_code)
        



import sqlite3
from typing import List

from popoll_backend.model.db.instrument import Instrument
from popoll_backend.model.db.user import User
from popoll_backend.model.db.user_instruments import UserInstruments
from popoll_backend.model.payload.users import Users
from popoll_backend.query.poll import PollQuery
from popoll_backend.query import fetchall


class GetUsers(PollQuery):
    
    def __init__(self, poll: str):
        super().__init__(poll)
    
    def process(self, cursor: sqlite3.Cursor) -> None:
        pass

    def buildResponse(self, cursor: sqlite3.Cursor) -> Users:
        return Users(fetchall(cursor.execute('SELECT * FROM users ORDER BY name COLLATE NOCASE'), User))


from tests import IntegrationTest

class TestUpdateUser(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru2_id, [self.instru3_id, self.instru1_id])['user']['id']
        
    def test_update_user(self):
        _json = self.update_user(self.user1_id, 'user1bis', self.instru3_id, [self.instru1_id, self.instru2_id])
        self.assertUserWithInstruments(_json, self.user1_id, 'user1bis', self.instru3_id, [self.instru1_id, self.instru2_id])

    def test_update_user_invalid(self):
        self.user2_id = self.create_user('user2', self.instru2_id, [self.instru3_id, self.instru1_id])['user']['id']
        _rs = self.update_user(self.user2_id, 'user1', self.instru3_id, [self.instru1_id, self.instru2_id], fail=True)
        self.assertEqual(409, _rs.status_code)


import datetime

from popoll_backend.query.all.delete_old_dates import dateBefore
from tests import IntegrationTest

class TestDeleteOldDates(IntegrationTest):
    
    today = datetime.date.today()

    def setUp(self):
        super().setUp()
        self.date1_id = self.create_date(date=dateBefore(years=2).isoformat(), time='15:00:00', title='2yearsBefore', end_time=None)['id']
        self.date2_id = self.create_date(date=dateBefore(years=1, days=1).isoformat(), time='15:00:00', title='2yearsAnd1DayBefore', end_time=None)['id']
        self.date3_id = self.create_date(date=dateBefore(years=1).isoformat(), time='15:00:00', title='1yearBefore', end_time=None)['id']
        self.date4_id = self.create_date(date=dateBefore(years=1, days=-1).isoformat(), time='15:00:00', title='1yearMinus1DayBefore', end_time=None)['id']
        
    def test_getDaysBefore_method(self):
        self.assertEqual(datetime.date.today().isoformat(), dateBefore(years=0, days=0).isoformat())
        self.assertEqual("2023-05-29", dateBefore(datetime.date(2024, 5, 29), 1, 0).isoformat())
        self.assertEqual("2024-05-28", dateBefore(datetime.date(2024, 5, 29), 0, 1).isoformat())
        self.assertEqual("2023-05-28", dateBefore(datetime.date(2024, 5, 29), 1, 1).isoformat())
        self.assertEqual("2023-05-30", dateBefore(datetime.date(2024, 5, 29), 1, -1).isoformat())
        
        
    def test_delete_old_dates(self):
        self.delete_old_date()
        _json = self.get_dates()
        self.assertEqual(2, len(_json['dates']))
        


from typing import List
from popoll_backend.model.db.date import Date
from popoll_backend.model.db.instrument import Instrument
from popoll_backend.model import Payload


class Dates(Payload):
    
    def __init__(self, dates: List[Date]):
        self.dates: List[Date] = dates


from tests import IntegrationTest

class TestGetDate(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru2_id, [self.instru1_id])['user']['id']
        self.user2_id = self.create_user('user2', self.instru1_id, [self.instru2_id])['user']['id']
        self.date1_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate', end_time='18:00:00')['id']
        self.answer1_id = self.create_answer(user_id=self.user1_id, date_id=self.date1_id)['id']
        self.answer2_id = self.create_answer(user_id=self.user2_id, date_id=self.date1_id)['id']
        self.update_answer(self.answer2_id, False)
        
    def test_get_date(self):
        _json = self.get_date(self.date1_id, False)
        self.assertDate(_json, self.date1_id, '2025-03-10', '15:00:00', '18:00:00', 'firstDate')
        self.assertEqual(None, _json.get('answers', None))

    def test_get_date_details(self):
        _json = self.get_date(self.date1_id, True)
        self.assertDate(_json['date'], self.date1_id, '2025-03-10', '15:00:00', '18:00:00', 'firstDate')
        
        self.assertEqual(4, len(_json['answers']))
        
        answer0 = _json['answers'][0]
        self.assertAnswer(answer0['answer'], self.answer1_id, self.user1_id, self.date1_id, True)
        self.assertUser(answer0['user'], self.user1_id, 'user1')
        self.assertInstrument(answer0['instrument'], self.instru2_id, self.INSTRU2, self.INSTRU2_RANK)
        self.assertTrue(answer0['is_main_instrument'])
        
        answer1 = _json['answers'][1]
        self.assertAnswer(answer1['answer'], self.answer1_id, self.user1_id, self.date1_id, True)
        self.assertUser(answer1['user'], self.user1_id, 'user1')
        self.assertInstrument(answer1['instrument'], self.instru1_id, self.INSTRU1, self.INSTRU1_RANK)
        self.assertFalse(answer1['is_main_instrument'])
        
        answer2 = _json['answers'][2]
        self.assertAnswer(answer2['answer'], self.answer2_id, self.user2_id, self.date1_id, False)
        self.assertUser(answer2['user'], self.user2_id, 'user2')
        self.assertInstrument(answer2['instrument'], self.instru1_id, self.INSTRU1, self.INSTRU1_RANK)
        self.assertTrue(answer2['is_main_instrument'])
        
        answer3 = _json['answers'][3]
        self.assertAnswer(answer3['answer'], self.answer2_id, self.user2_id, self.date1_id, False)
        self.assertUser(answer3['user'], self.user2_id, 'user2')
        self.assertInstrument(answer3['instrument'], self.instru2_id, self.INSTRU2, self.INSTRU2_RANK)
        self.assertFalse(answer3['is_main_instrument'])
        


import sqlite3

import flask

from popoll_backend.model.db.answer import Answer
from popoll_backend.query.poll import PollQuery
from popoll_backend.query import fetchlast


class GetSearchAnswer(PollQuery):
    
    userId: int
    dateId: int
    
    def __init__(self, poll: str, userId: int, dateId: int):
        super().__init__(poll)
        self.userId = userId
        self.dateId = dateId

    def process(self, cursor: sqlite3.Cursor) -> None:
        pass
    
    def buildResponse(self, cursor: sqlite3.Cursor) -> Answer:
        return fetchlast(cursor.execute('SELECT * FROM answers WHERE user_id=? AND date_id=?', (self.userId, self.dateId)), Answer)


import os
import sqlite3
from typing import Optional
import flask

from popoll_backend.model import Payload
from popoll_backend.query import _Query


class PollQuery(_Query):
    
    fail_if_db_exists: bool = False
    fail_if_db_not_exists: bool = True
    
    def __init__(self, poll: str):
        self.poll = poll
        self.db_file = f'{poll}.db'
        if self.fail_if_db_exists and os.path.exists(self.db_file):
            flask.abort(409, f'The poll [{poll}] already exists.')
        if self.fail_if_db_not_exists and not os.path.exists(self.db_file):
            flask.abort(400, f'The poll [{poll}] does not exist.')
    
    def run(self, cursor: Optional[sqlite3.Cursor]=None) -> Payload:
        if cursor:
            return self._run(cursor)
        else:
            with sqlite3.connect(self.db_file) as connection:
                _cursor: sqlite3.Cursor = connection.cursor()
                _cursor.execute("PRAGMA foreign_keys = ON")
                # _cursor.execute("ATTACH DATABASE 'instruments.db' AS instruments;")
                return self._run(_cursor)
            
    def _run(self, cursor: sqlite3.Cursor) -> Payload:
        try:
            self.process(cursor)
            return self.buildResponse(cursor)
        except sqlite3.Error as e:
            print(e)
            self.error(e)
            if e.sqlite_errorcode == sqlite3.SQLITE_CONSTRAINT_FOREIGNKEY:
                flask.abort(400, f'ForeignKey error. You refer to a not existing resource')
            if e.sqlite_errorcode == sqlite3.SQLITE_ERROR:
                flask.abort(500, 'Unknown DB error')
            flask.abort(500, 'Unknown server error')
    
    def process(self, cursor: sqlite3.Cursor) -> None:
        raise NotImplementedError()
    
    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        raise NotImplementedError()
    
    def error(self, e: sqlite3.Error) -> None:
        pass


import sqlite3

from popoll_backend.model.db.option import Option
from popoll_backend.query.poll import PollQuery


class GetPoll(PollQuery):
    
    def __init__(self, poll: str):
        super().__init__(poll)

    def process(self, cursor: sqlite3.Cursor) -> None:
        pass
    
    def buildResponse(self, cursor: sqlite3.Cursor) -> Option:
        return Option(cursor.execute('SELECT * FROM options').fetchone())


import sqlite3

from popoll_backend.model import Payload
from popoll_backend.model.db.date import Date
from popoll_backend.query.poll import PollQuery
from popoll_backend.query import fetchlast


class GetDate(PollQuery):
    
    id: int
    
    def __init__(self, poll: str, id: int):
        super().__init__(poll)
        self.id = id

    def process(self, cursor: sqlite3.Cursor) -> None:
        pass

    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        return fetchlast(cursor.execute('SELECT * FROM dates WHERE id=? ORDER BY date, time', (self.id,)), Date)


from tests import IntegrationTest

class TestUpdatePoll(IntegrationTest):

    def test_db_creation(self):
        _json = self.update_poll('TESTTESTTEST', '#88fb00')
        self.assertEqual('TESTTESTTEST', _json['name'])
        self.assertEqual('#88fb00', _json['color'])
        
        _json = self.get_poll()
        self.assertEqual('TESTTESTTEST', _json['name'])
        self.assertEqual('#88fb00', _json['color'])



import datetime
import sqlite3
from typing import List, Tuple

from popoll_backend.model import Payload
from popoll_backend.model.db.date import Date
from popoll_backend.model.payload.empty import Empty
from popoll_backend.query.all import Query
from popoll_backend.query import fetchall

def dateBefore(day=datetime.date.today(), years: int=0, days: int=0):
    today = day.isoformat()
    beforeYears = today[0:3] + str(int(today[3])-years) + today[4:] # big hack to replace year in string to get rid of bissectil years
    return (datetime.date.fromisoformat(beforeYears) - datetime.timedelta(days=days))

class DeleteOldDates(Query):
    
    past = dateBefore(years=1)
    
    def process(self, db: str, cursor: sqlite3.Cursor) -> None:
        dates: List[Date] = fetchall(cursor.execute('SELECT * FROM dates'), Date)
        old_dates_ids: List[Tuple[int]] = [(date.id,) for date in dates if date.isOlder(self.past)]
        cursor.executemany('DELETE FROM dates WHERE id=?', old_dates_ids)
    
    def buildResponse(self, db: str, cursor: sqlite3.Cursor) -> Payload:
        return Empty()
    
    def mergeResponses(self, answers: List[Payload]) -> Payload:
        return Empty()



import sqlite3

from popoll_backend.model.db.answer import Answer
from popoll_backend.model.db.date import Date
from popoll_backend.model.db.instrument import Instrument
from popoll_backend.model.db.user import User
from popoll_backend.model.db.user_instruments import UserInstruments
from popoll_backend.model.payload.date_details import DateDetails
from popoll_backend.query.poll import PollQuery
from popoll_backend.query import fetchall, fetchlast


class GetDateDetails(PollQuery):
    
    id: int
    
    def __init__(self, poll: str, id: int):
        super().__init__(poll)
        self.id = id

    def process(self, cursor: sqlite3.Cursor) -> None:
        pass

    def buildResponse(self, cursor: sqlite3.Cursor) -> DateDetails:
        globally_used_instruments = [row[0] for row in cursor.execute('SELECT DISTINCT instrument_id FROM user_instruments').fetchall()]
        return DateDetails(
            date=fetchlast(cursor.execute('SELECT * FROM dates where id=?', (self.id,)), Date), 
            answers=fetchall(cursor.execute('SELECT * FROM answers where date_id=?', (self.id,)), Answer),
            users=fetchall(cursor.execute('SELECT * FROM users ORDER BY name COLLATE NOCASE'), User),     
            instruments=[instr for instr in fetchall(cursor.execute('SELECT * FROM instruments ORDER BY rank'), Instrument) if instr.id in globally_used_instruments],
            user_instruments=fetchall(cursor.execute('SELECT * FROM user_instruments'), UserInstruments)
        )


import sqlite3
from typing import List

from popoll_backend.model.db.instrument import Instrument
from popoll_backend.model.payload.instruments import Instruments
from popoll_backend.query.poll import PollQuery
from popoll_backend.query import fetchall


class GetInstruments(PollQuery):
    
    instruments: List[Instrument]
    
    def __init__(self, poll: str):
        super().__init__(poll)

    def process(self, cursor: sqlite3.Cursor) -> None:
        pass
            
    def buildResponse(self, cursor: sqlite3.Cursor) -> Instruments:
        globally_used_instruments = [row[0] for row in cursor.execute('SELECT DISTINCT instrument_id FROM user_instruments').fetchall()]
        return Instruments([instr for instr in fetchall(cursor.execute('SELECT * FROM instruments ORDER BY rank'), Instrument) if instr.id in globally_used_instruments])


from tests import IntegrationTest

class TestGetSession(IntegrationTest):
    
    SESSION = 'XXXXX'

    def setUp(self):
        super().setUp()
        
    def test_create_session_noUser(self):
        _rs = self.create_session(self.SESSION, 1, fail=True)
        self.assertEqual(400, _rs.status_code)
        
    def test_create_session(self):
        self.user1_id = self.create_user('user1', self.instru1_id, [])['user']['id']
        _json = self.create_session(self.SESSION, self.user1_id)
        self.assertSession(_json, self.SESSION, self.user1_id)
        


import sqlite3

from popoll_backend.model.db.instrument import Instrument
from popoll_backend.query.poll import PollQuery
from popoll_backend.query import fetchlast


class GetInstrument(PollQuery):
    
    id: int
    
    def __init__(self, poll: str, id: int):
        super().__init__(poll)
        self.id = id
    
    def process(self, cursor: sqlite3.Cursor) -> None:
        pass
        
    def buildResponse(self, cursor: sqlite3.Cursor) -> Instrument:
        return fetchlast(cursor.execute('SELECT * FROM instruments WHERE id=?', (self.id,)), Instrument)


import datetime
import sqlite3

from popoll_backend.model.db.session import Session
from popoll_backend.query.poll import PollQuery
from popoll_backend.query import fetchlast


class CreateSession(PollQuery):
    
    id: int
    
    def __init__(self, poll: str, id: str, user_id: int):
        super().__init__(poll)
        self.id = id
        self.user_id = user_id

    def process(self, cursor: sqlite3.Cursor) -> None:
        cursor.execute('DELETE FROM sessions WHERE session_id=?', (self.id,))
        cursor.execute('INSERT INTO sessions(session_id, user_id, datetime) VALUES(?, ?, ?)', (self.id, self.user_id, self.getNow())).lastrowid

    def buildResponse(self, cursor: sqlite3.Cursor) -> Session:
        return fetchlast(cursor.execute('SELECT * FROM sessions WHERE session_id=?', (self.id,)), Session)
    
    def getNow(self) -> str:
        return datetime.datetime.now().isoformat(sep='T', timespec='auto')


import contextlib
import datetime
import flask
import json

from popoll_backend.model import Payload

class History(Payload):
    
    def __init__(self, request: flask.Request, response, kwargs):
        self.date = datetime.datetime.now().isoformat(sep='T', timespec='auto')
        self.method = request.method
        self.path = request.path
        self.ip = request.headers.get('Host', None)
        self.sessionId = request.headers.get('SessionId', None)
        self.queryParams = kwargs
        self.body = request.data
        with contextlib.suppress(Exception):
            self.body_json = json.loads(request.data)
        self.response = response


import sqlite3

from popoll_backend.model.db.answer import Answer
from popoll_backend.model.db.date import Date
from popoll_backend.model.db.user import User
from popoll_backend.model.payload.user_with_instruments import UserWithInstruments
from popoll_backend.model.payload.user_details import UserDetails
from popoll_backend.query.poll import PollQuery
from popoll_backend.query import fetchall, fetchlast


class GetUserDetails(PollQuery):
    
    id: int
    
    def __init__(self, poll: str, id: int):
        super().__init__(poll)
        self.id = id
    
    def process(self, cursor: sqlite3.Cursor) -> None:
        pass
        
    def buildResponse(self, cursor: sqlite3.Cursor) -> UserWithInstruments:
        return UserDetails(
            user=fetchlast(cursor.execute('SELECT * FROM users WHERE id=?', (self.id,)), User), 
            answers=fetchall(cursor.execute('SELECT * FROM answers WHERE user_id=?', (self.id,)), Answer),
            dates=fetchall(cursor.execute('SELECT * FROM dates ORDER BY date, time'), Date)
        )

        


from tests import IntegrationTest

class TestGetUser(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru2_id, [self.instru3_id, self.instru1_id])['user']['id']
        
    def test_get_user(self):
        _json = self.get_user(self.user1_id, False)
        self.assertUserWithInstruments(_json, self.user1_id, 'user1', self.instru2_id, [self.instru1_id, self.instru3_id])

    def test_get_user_details(self):
        self.date1_id = self.create_date(date='2028-03-10', time='15:00:00', title='firstDate', end_time='18:00:00')['id']
        self.date2_id = self.create_date(date='2022-03-10', time='15:00:00', title='oldDate', end_time='18:00:00')['id']
        self.answer1_id = self.create_answer(user_id=self.user1_id, date_id=self.date1_id)['id']
        _json = self.get_user(self.user1_id, True)
        self.assertUser(_json['user'], self.user1_id, 'user1')
        self.assertEqual(2, len(_json['dates']))
        self.assertDate(_json['dates'][0]['date'], self.date2_id, '2022-03-10', '15:00:00', '18:00:00', 'oldDate', False, True)
        self.assertDate(_json['dates'][1]['date'], self.date1_id, '2028-03-10', '15:00:00', '18:00:00', 'firstDate', False, False)
        self.assertAnswer(_json['dates'][1]['answer'], self.answer1_id, self.user1_id, self.date1_id, True)
        
    def test_get_user_details_with_old_dates(self):
        self.date1_id = self.create_date(date='2028-03-10', time='15:00:00', title='firstDate', end_time='18:00:00')['id']
        self.date2_id = self.create_date(date='2022-03-10', time='15:00:00', title='oldDate', end_time='18:00:00')['id']
        self.answer1_id = self.create_answer(user_id=self.user1_id, date_id=self.date1_id)['id']
        _json = self.get_user(self.user1_id, True)
        self.assertEqual(2, len(_json['dates']))


import json
import jsonpickle

from typing import Any, List

class Payload:
    
    isEmpty = False
    
    def toJSON(self):
        return json.loads(jsonpickle.encode(self, unpicklable=False))
    
    def toDict(self, llist: List[Any]):
        return {i.id: i for i in llist}


import sqlite3

from popoll_backend.model.db.session import Session
from popoll_backend.query.poll import PollQuery
from popoll_backend.query import fetchlast


class GetSession(PollQuery):
    
    id: int
    session: Session
    
    def __init__(self, poll: str, id: str):
        super().__init__(poll)
        self.id = id

    def process(self, cursor: sqlite3.Cursor) -> None:
        pass

    def buildResponse(self, cursor: sqlite3.Cursor) -> Session:
        return fetchlast(cursor.execute('SELECT * FROM sessions WHERE session_id=?', (self.id,)), Session)




import sqlite3
import toml
from typing import List

import requests

from popoll_backend.model import Payload
from popoll_backend.model.payload.empty import Empty
from popoll_backend.model.payload.id_payload import IdPayload
from popoll_backend.query.all import Query


class UploadPolls(Query):
  
  status_code: int
  
  def __init__(self):
    with open("popoll_backend.toml", "rb") as f:
      config = toml.loads(f)
    self.drive_id = config['kdrive']['drive_id']
    self.token = config['kdrive']['token']
    self.folder_id = self.get_folder_id(config['kdrive']['folder'], self.drive_id, self.token)

  def process(self, db: str, cursor: sqlite3.Cursor) -> None:
    with open(f'{db}.db', mode='rb') as file:
      data = file.read()
        
    res = requests.post(
      url=f'https://api.infomaniak.com/3/drive/{self.drive_id}/upload?conflict=version&file_name={db}.db&total_size={len(data)}&directory_id={self.folder_id}', 
      data=data, 
      headers=self.headers(self.token, 'application/octet-stream')
    )
    print(f'[{res.status_code}] {res.json()}')
    self.status_code = res.status_code
      
  def buildResponse(self, db: str, cursor: sqlite3.Cursor) -> IdPayload:
      return IdPayload(self.status_code)
      
  def mergeResponses(self, answers: List[IdPayload]) -> IdPayload:
      return IdPayload(max([a.id for a in answers]) if len(answers) > 0 else 500)

  def get_folder_id(self, folder, drive_id, token):
    rs = requests.get(url=f'https://api.infomaniak.com/3/drive/{drive_id}/files/search', headers=self.headers(token, 'application/json'))
    return [data['id'] for data in (rs.json())['data'] if data['name'] == folder][0]
  
  def headers(self, token, contentType):
    return {
    'Authorization': f'Bearer {token}',
    'Content-Type': contentType
    }


from tests import IntegrationTest

class TestCreateDate(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.date1_id = self.create_date(date='2025-03-12', time='15:00:00', title='firstDate', end_time=None)['id']
        
    def test_create_date(self):
        self.assertEqual(1, len(self.get_dates()['dates']))
        self.date2_id = self.create_date(date='2025-03-10', title='secondDate', time=None, end_time=None)['id']
        _json = self.get_dates()
        self.assertEqual(2, len(_json['dates']))
        self.assertDates(_json['dates'], 0, self.date2_id, '2025-03-10', None, None, 'secondDate')
        
    def test_create_date_only_end_time(self):
        self.assertEqual(1, len(self.get_dates()['dates']))
        self.date2_id = self.create_date(date='2025-03-10', title='secondDate', time=None, end_time='18:00:00')['id']
        _json = self.get_dates()
        self.assertEqual(2, len(_json['dates']))
        self.assertDates(_json['dates'], 0, self.date2_id, '2025-03-10', None, '18:00:00', 'secondDate')



from typing import List, Optional
from popoll_backend.model import Payload


class Poll(Payload):
    
    def __init__(self, id: str, name: str):
        self.id = id
        self.name = name

class Polls(Payload):
    
    def __init__(self, polls: List[Poll]):
        self.polls: List[Poll] = polls



import datetime
import sqlite3
from typing import Optional

from popoll_backend.model.db.date import Date
from popoll_backend.query.poll import PollQuery
from popoll_backend.query import fetchlast


class UpdateDate(PollQuery):
    
    id: int
    title: str
    date: datetime.date
    time: Optional[datetime.time]
    end_time: Optional[datetime.time]
    is_frozen: bool
    
    def __init__(self, poll: str, id: int, title: str, date: datetime.date, time: Optional[datetime.time], end_time: Optional[datetime.time], is_frozen: bool):
        super().__init__(poll)
        self.id = id
        self.title = title
        self.date = date
        self.time = time
        self.end_time = end_time
        self.is_frozen = is_frozen
        
    def process(self, cursor: sqlite3.Cursor) -> None:
        cursor.execute('UPDATE dates SET title=?, date=?, time=?, end_time=?, is_frozen=? WHERE id=?', (self.title, self.date, self.time, self.end_time, self.is_frozen, self.id))
    
    def buildResponse(self, cursor: sqlite3.Cursor) -> Date:
        return fetchlast(cursor.execute('SELECT * FROM dates WHERE id=?', (self.id,)), Date)


import sqlite3
from typing import Optional

import flask

from popoll_backend.model import Payload
from popoll_backend.model.db.answer import Answer
from popoll_backend.query.poll import PollQuery
from popoll_backend.query import fetchlast, get_date_id, is_date_frozen


class UpdateAnswer(PollQuery):
    
    id: int
    response: Optional[bool]
    
    def __init__(self, poll:str, id: int, response: Optional[bool]):
        super().__init__(poll)
        self.id = id
        self.response = response
    
    def process(self, cursor: sqlite3.Cursor):
        if is_date_frozen(cursor, get_date_id(cursor, self.id)):
            flask.abort(403, 'Date is frozen. Cannot modify')
        cursor.execute('UPDATE answers SET response=? WHERE id=?', (self.response, self.id))
    
    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        return fetchlast(cursor.execute('SELECT * FROM answers WHERE id=?', (self.id,)), Answer)


from tests import IntegrationTest

class TestGetUsers(IntegrationTest):

    def setUp(self):
        super().setUp()
        
    def test_get_users_sorted(self):
        self.create_user('user2', self.instru1_id, [])
        self.create_user('user1', self.instru1_id, [])
        _json = self.get_users()
        self.assertEqual('undefined', _json.get('dates', 'undefined'))
        self.assertEqual('undefined', _json.get('answers', 'undefined'))
        self.assertEqual(2, len(_json['users']))
        self.assertUsers(_json['users'], 0, 2, 'user1')
        self.assertUsers(_json['users'], 1, 1, 'user2')

    def test_get_users_case(self):
        self.create_user('user1', self.instru1_id, [])
        self.create_user('User2', self.instru1_id, [])
        self.create_user('user3', self.instru1_id, [])
        _json = self.get_users()
        self.assertEqual('undefined', _json.get('dates', 'undefined'))
        self.assertEqual('undefined', _json.get('answers', 'undefined'))
        self.assertEqual(3, len(_json['users']))
        self.assertUsers(_json['users'], 0, 1, 'user1')
        self.assertUsers(_json['users'], 1, 2, 'User2')
        self.assertUsers(_json['users'], 2, 3, 'user3')



import sqlite3

from popoll_backend.model.db.option import Option
from popoll_backend.query.poll import PollQuery


class UpdatePoll(PollQuery):
    
    name: str
    color: str
    
    def __init__(self, poll: str, name: str, color: str):
        super().__init__(poll)
        self.name = name
        self.color = color
        
    def process(self, cursor: sqlite3.Cursor) -> None:
        cursor.execute('UPDATE options SET name=?, color=?', (self.name, self.color))
    
    def buildResponse(self, cursor: sqlite3.Cursor) -> Option:
        return Option(cursor.execute('SELECT * FROM options').fetchone())


from typing import Any, List
from popoll_backend.model import Payload
from popoll_backend.model.db.user import User
from popoll_backend.model.db.instrument import Instrument
from popoll_backend.model.db.user_instruments import UserInstruments


class UserWithInstruments(Payload):
    
    def __init__(self, user: User, instruments: List[Instrument], user_instruments: List[UserInstruments]):
        self.user = user
        self.main_instrument: Instrument = self.getInstrument(instruments, self.getInstrumentIds(user_instruments, user.id, True)[0])
        self.instruments: List[Instrument] = self.getInstruments(instruments, self.getInstrumentIds(user_instruments, user.id, False))
        
    def getInstrumentIds(self, user_instruments: List[UserInstruments], user_id: int, is_main: bool):
        return [user_instrument.instrument_id for user_instrument in user_instruments if user_instrument.user_id == user_id and user_instrument.is_main == is_main]
    
    def getInstruments(self, instruments: List[Instrument], ids: List[int]):
        return [instrument for instrument in instruments if instrument.id in ids]
    
    def getInstrument(self, instruments: List[Instrument], id: int):
        return [instrument for instrument in instruments if instrument.id == id][0]



from typing import List
from popoll_backend.model import Payload
from popoll_backend.model.payload.user_with_instruments import UserWithInstruments


class UsersPayloadDetails(Payload):
    
    def __init__(self, users: List[UserWithInstruments]):
        self.users: List[UserWithInstruments] = users


import sqlite3
from typing import List, Union

from popoll_backend.model import Payload
from popoll_backend.model.payload.empty import Empty
from popoll_backend.model.payload.polls import Poll, Polls
from popoll_backend.query.all import Query


class GetAllSession(Query):
    
    id: str
    
    def __init__(self, id: str):
        self.id = id
        
    def process(self, db: str, cursor: sqlite3.Cursor) -> None:
        pass
        
    def buildResponse(self, db: str, cursor: sqlite3.Cursor) -> Union[Poll, Empty]:
        if cursor.execute('SELECT COUNT(*) FROM sessions WHERE session_id=?', (self.id,)).fetchone()[0] > 0:
            return Poll(db, cursor.execute('SELECT name FROM options').fetchone()[0])
        else:
            return Empty()
        
    def mergeResponses(self, answers: List[Payload]) -> Payload:
        return Polls([a for a in answers if not a.isEmpty])
    
    



from typing import List
from popoll_backend.model.db.user import User
from popoll_backend.model import Payload


class Users(Payload):
    
    def __init__(self, users: List[User]):
        self.users: List[User] = users


from tests import IntegrationTest

class TestGetPoll(IntegrationTest):

    def test_db_creation(self):
        _json = self.get_instruments(False)
        self.assertEqual(9, len(_json['instruments']))
        self.assertEqual(self.instru1_id, _json['instruments'][0]['id'])
        self.assertEqual(self.INSTRU1, _json['instruments'][0]['name'])
        self.assertEqual(self.instru2_id, _json['instruments'][1]['id'])
        self.assertEqual(self.INSTRU2, _json['instruments'][1]['name'])
        self.assertEqual(self.instru3_id, _json['instruments'][2]['id'])
        self.assertEqual(self.INSTRU3, _json['instruments'][2]['name'])

    def test_get_poll(self):
        _json = self.get_poll()
        self.assertEqual('TESTTEST', _json['name'])
        self.assertEqual('#ff8b00', _json['color'])


from tests import IntegrationTest

class TestUpdateDate(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.date1_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate', end_time='18:00:00')['id']
        
    def test_update_date(self):
        _json = self.udpate_date(self.date1_id, date='2025-03-10', title='firstDate', time=None, end_time=None)
        self.assertDate(_json, self.date1_id, '2025-03-10', None, None, 'firstDate')
        _json = self.get_date(self.date1_id, False)
        self.assertDate(_json, self.date1_id, '2025-03-10', None, None, 'firstDate')

    def test_update_freeze_date(self):
        _json = self.udpate_date(self.date1_id, date='2025-03-10', time='15:00:00', title='firstDate', end_time='18:00:00', is_frozen=True)
        self.assertTrue(_json['is_frozen'])
        
        self.user1_id = self.create_user('user1', self.instru2_id, [self.instru1_id])['user']['id']
        _rs = self.create_answer(self.user1_id, self.date1_id, fail=True)
        self.assertEqual(403, _rs.status_code)


import sqlite3
from typing import List

from popoll_backend.model.db.date import Date
from popoll_backend.model.payload.dates import Dates
from popoll_backend.query.poll import PollQuery
from popoll_backend.query import fetchall


class GetDates(PollQuery):
    
    dates: List[Date]
    
    def __init__(self, poll: str):
        super().__init__(poll)

    def process(self, cursor: sqlite3.Cursor):
        pass
        
    def buildResponse(self, cursor: sqlite3.Cursor) -> Dates:
        return Dates(fetchall(cursor.execute('SELECT * FROM dates ORDER BY date, time'), Date))


import flask
import sqlite3

from typing import List

from popoll_backend.model import Payload


def fetchall(cursor: sqlite3.Cursor, ttype: type) -> List[type]:
        return [ttype(row) for row in cursor.fetchall()]

def fetchlast(cursor: sqlite3.Cursor, ttype: type) -> type:
        res = fetchall(cursor, ttype)
        if len(res) > 0:
            return res[-1]
        flask.abort(404, 'Not found')

def is_date_frozen(cursor: sqlite3.Cursor, date_id: int) -> bool:
    res = cursor.execute('SELECT is_frozen FROM dates WHERE id=?', (date_id,)).fetchone()
    if res != None and len(res) > 0:
        return res[0]
    else:
        flask.abort(400, 'Not found')

def get_date_id(cursor:sqlite3.Cursor, answer_id: int) -> int:
    res = cursor.execute('SELECT date_id FROM answers WHERE id=?', (answer_id,)).fetchone()
    if res != None and len(res) > 0:
        return res[0]
    else:
        flask.abort(404, 'Not found')

class _Query():
    
    def run(self) -> Payload:
        raise NotImplementedError()


    



from tests import DB_NAME, IntegrationTest

class TestGetSession(IntegrationTest):
    
    DB_2 = 'test_integration_2'
    DB_3 = 'test_integration_3'
    
    SESSION = 'XXXXX'

    def setUp(self):
        super().setUp()
        self.user1_id: int = self.create_user('user1', self.instru1_id, [])['user']['id']
        self.create_session(self.SESSION, self.user1_id)
        
        self.create_poll(self.DB_2, self.DB_2, '#ff8b01')
        self.create_user('user1', self.instru1_id, [], db=self.DB_2)
        self.create_session(self.SESSION, self.user1_id, db=self.DB_2)
        
        self.create_poll(self.DB_3, self.DB_3, '#ff8b02')
        
    def test_get_all_session(self):
        _json = self.get_all_session(self.SESSION)
        self.assertEqual(2, len(_json['polls']))
        self.assertPoll(_json['polls'][0], DB_NAME, 'TESTTEST')
        self.assertPoll(_json['polls'][1], self.DB_2, self.DB_2)

    def tearDown(self):
        self.tearDown_db(self.DB_2)
        self.tearDown_db(self.DB_3)


from tests import IntegrationTest

class TestGetDates(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.date1_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate', end_time=None)['id']
        self.date2_id = self.create_date(date='2025-03-12', title='secondDate', time=None, end_time=None)['id']
        self.date3_id = self.create_date(date='2025-03-11', time='15:00:00', title='thirdDate', end_time='18:00:00')['id']
        self.date4_id = self.create_date(date='2025-03-11', title='fourthDate', end_time='18:00:00', time=None)['id']
        
    def test_get_dates(self):
        _json = self.get_dates()
        self.assertEqual(4, len(_json['dates']))
        self.assertDates(_json['dates'], 0, self.date1_id, '2025-03-10', '15:00:00', None, 'firstDate')
        self.assertDates(_json['dates'], 1, self.date4_id, '2025-03-11', None, '18:00:00', 'fourthDate')
        self.assertDates(_json['dates'], 2, self.date3_id, '2025-03-11', '15:00:00', '18:00:00', 'thirdDate')
        self.assertDates(_json['dates'], 3, self.date2_id, '2025-03-12', None, None, 'secondDate')



from typing import Any, List
from popoll_backend.model import Payload


class Instrument(Payload):
    
    def __init__(self, data: List[Any]):
        self.id: int = data[0]
        self.name: str = data[1]
        self.rank: int = data[2]

    @classmethod
    def create_table(cls):
        return """CREATE TABLE IF NOT EXISTS instruments (
            id integer PRIMARY KEY,
            name text NOT NULL UNIQUE,
            rank number NOT NULL UNIQUE
        );
        """



import sqlite3

from popoll_backend.model.db.instrument import Instrument
from popoll_backend.model.db.user import User
from popoll_backend.model.db.user_instruments import UserInstruments
from popoll_backend.model.payload.user_with_instruments import UserWithInstruments
from popoll_backend.query.poll import PollQuery
from popoll_backend.query import fetchall, fetchlast


class GetUserWithInstruments(PollQuery):
    
    id: int
    
    def __init__(self, poll: str, id: int):
        super().__init__(poll)
        self.id = id
    
    def process(self, cursor: sqlite3.Cursor) -> None:
        pass
        
    def buildResponse(self, cursor: sqlite3.Cursor) -> UserWithInstruments:
        return UserWithInstruments(
            user=fetchlast(cursor.execute('SELECT * FROM users WHERE id=?', (self.id,)), User), 
            instruments=fetchall(cursor.execute('SELECT * FROM instruments ORDER BY rank'), Instrument),
            user_instruments=fetchall(cursor.execute('SELECT * FROM user_instruments WHERE user_id=?', (self.id,)), UserInstruments)
        )
        
        
        


import glob
import sqlite3
from typing import List
from popoll_backend.model import Payload
from popoll_backend.query import _Query


class Query(_Query):
    
    def run(self) -> Payload:
        answers: List[Payload] = []
        for db in sorted(glob.glob('*.db')):
            # We do not want to break in case a db is incorrect
            try:
                with sqlite3.connect(db) as connection:
                    cursor: sqlite3.Cursor = connection.cursor()
                    self.process(db[0:-3], cursor)
                    answers.append(self.buildResponse(db[0:-3], cursor))
            except Exception as e:
                print(e)
        return self.mergeResponses(answers)
    
    def process(self, db: str, cursor: sqlite3.Cursor) -> None:
        raise NotImplementedError()
    
    def buildResponse(self, db: str, cursor: sqlite3.Cursor) -> Payload:
        raise NotImplementedError()
    
    def mergeResponses(self, answers: List[Payload]) -> Payload:
        raise NotImplementedError()


from tests import IntegrationTest

class TestGetInstruments(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru1_id, [self.instru2_id])['user']['id']
        self.user2_id = self.create_user('user2', self.instru2_id, [self.instru1_id])['user']['id']

    def test_get_instruments(self):
        _json = self.get_instruments(False)
        self.assertEqual(9, len(_json['instruments']))
        self.assertTrue(self.INSTRU1 in [i['name'] for i in _json['instruments']])
        self.assertTrue(self.INSTRU2 in [i['name'] for i in _json['instruments']])
        self.assertTrue(self.INSTRU3 in [i['name'] for i in _json['instruments']])
        
    def test_get_instruments_used_only(self):
        _json = self.get_instruments(True)
        self.assertEqual(2, len(_json['instruments']))
        self.assertTrue(self.INSTRU1 in [i['name'] for i in _json['instruments']])
        self.assertTrue(self.INSTRU2 in [i['name'] for i in _json['instruments']])
        
    def test_get_instruments_used_only_all_used(self):
        self.update_user(self.user2_id, 'user2', self.instru2_id, [self.instru1_id, self.instru3_id])
        _json = self.get_instruments(True)
        self.assertEqual(3, len(_json['instruments']))

